from __future__ import print_function
import sys

from app.singleton.singleton import Singleton
import json, os
from ..logger.logger import Logger
#


class Configurator(object):
    __metaclass__ = Singleton
    host = ""
    flow_output_format_int = []
    flow_output_format_ext = []
    component_timeout = 0
    logger = Logger()


    def __init__(self):
        with open('store/configuration.json') as conFile:
            config = json.load(conFile)
            self.host = config['host']
            self.flow_output_format_int = config['flowFormatInternal']
            self.flow_output_format_ext = config['flowFormatExternal']

    def getCreddentials(self):
        with open('store/configuration.json') as conFile:
            config = json.load(conFile)
        return {
            'username': config['username'],
            'password': config['password']
        }

    def getConfig(self):
        with open('store/configuration.json') as conFile:
            config = json.load(conFile)
        config.pop('password')
        return config

    def setSecretKey(self, sk):
        with open('store/configuration.json') as conFile:
            config = json.load(conFile)
        config['secretKey'] = sk
        with open('store/configuration.json', 'w') as conFile:
            json.dump(config, conFile)


    def setConfig(self, config):
        with open('store/configuration.json') as conFile:
            configTmp = json.load(conFile)
        oldHost = configTmp['host']
        oldSecretKey = configTmp['secretKey']
        config['secretKey'] = oldSecretKey

        self.host = config['host']
        self.flow_output_format_int = config['flowFormatInternal']
        self.flow_output_format_ext = config['flowFormatExternal']

        if (config['password'] == False):
            self.logger.log(2, 'configurator.setConfig', 'Setting new password to config')
            with open('store/configuration.json') as conFile:
                oldConfig = json.load(conFile)
            config['password'] = oldConfig['password']

        #print("Oldhost: ", oldHost, file=sys.stderr)
        #print("Newhost: ", config['host'], file=sys.stderr)

        try:
            with open('store/configuration.json', 'w') as conFile:
                json.dump(config, conFile)
            self.logger.updateLogLevel()
            self.logger.log(1, 'configurator.setConfig', 'New config successfully stored')

            if oldHost != self.host:
                self.logger.log(1, 'configurator.setConfig', 'Detected change of hostname')
                from ..connector.connector import Connector
                Connector().getNewToken()
                clearWidgetsConf = [{},
                    {
                    "perspective_name": "",
                    "perspective_id": 0,
                    "period": 0
                    },
                    {
                    "perspective_name": "",
                    "perspective_id": 0,
                    "period": 0
                    },
                    {
                    "perspective_name": "",
                    "perspective_id": 0,
                    "period": 0
                    },
                    {
                    "perspective_name": "",
                    "perspective_id": 0,
                    "period": 0
                    },
                    {
                    "perspective_name": "",
                    "perspective_id": 0,
                    "period": 0
                    }
                ]

                with open('store/widgets.json', 'w') as widFile:
                    json.dump(clearWidgetsConf, widFile)
            self.logger.log(1, 'configurator.setConfig', 'Widgets are reseted')

        except:
            return "", 400
        return "", 200

    def getWidgetsInfo(self):
        with open('store/widgets.json', 'r') as f:
            config = json.load(f)
        self.logger.log(2, 'configurator.getWidgetsInfo', 'Retrieving widgets config: '+str(config))
        return config

    def setWidgetInfoById(self, widgetId, perspId, perspName):
        #print("PerspName: ", perspName, type(perspName), file=sys.stderr)
        self.logger.log(1, 'configurator.setWidgetInfoById', 'Setting new perspective for widget'+str(widgetId)+': '+str(perspId)+'/'+perspName)
        with open('store/widgets.json', 'r') as f:
            config = json.load(f)
        config[widgetId]['perspective_id'] = perspId
        config[widgetId]['perspective_name'] = perspName
        with open('store/widgets.json', 'w') as conFile:
            json.dump(config, conFile)

    def setWidgetPeriodById(self, widgetId, period):
        self.logger.log(1, 'configurator.setWidgetPeriodById', 'Setting new period for widget' + str(widgetId) + ': ' + str(period))
        with open('store/widgets.json', 'r') as f:
            config = json.load(f)
        config[widgetId]['period'] = period
        with open('store/widgets.json', 'w') as conFile:
            json.dump(config, conFile)