from __future__ import print_function
import sys

from ..singleton.singleton import Singleton
from ..configurator.configurator import Configurator
from ..logger.logger import Logger
import requests, json, time


class Connector(object):
    __metaclass__ = Singleton
    #__host = "https://flowmon.iseco.local"
    __conf = Configurator()
    __host = __conf.getConfig()['host']
    #__host = "http://10.0.20.3"
    __token = ""
    __tokenType = ""
    __expires = ""
    logger = Logger()


    def __init__(self):
        #print('initing connector', file=sys.stderr)
        self.logger.log(2, 'connector.__init__', 'Initing connector...')
        self.__host = self.__conf.getConfig()['host']
        #print('result: self.__host: ', self.__host, file=sys.stderr)
        self.logger.log(2, 'connector.__init__', 'Host:'+self.__host)
        self.logger.log(1, 'connector.__init__', 'Connector initialized')

    def initialize(self):
        try:
            self.getNewToken()
        except Exception as e:
            print('Exception while getNewToken ', e, file=sys.stderr)
            return False
        return True

    def __checkTokenExpiration(self):
        #returns true of ok, false if expired
        #print('+++ Expire control. Now:', time.time(), ' Expires: ', self.__expires, " ", (self.__expires > time.time()), file=sys.stderr)
        self.logger.log(2, 'connector.__checkTokenExpiration', 'Token is valid: ' + str(self.__expires > time.time()))
        if self.__expires < time.time() or self.__token == "":
            print("ret false - token not ok " + 'Token is invalid because ' + str(self.__expires) + '>' + str(time.time()), file=sys.stderr)
            self.logger.log(2, 'connector.__checkTokenExpiration','Token is invalid because ' + str(self.__expires) + '>' + str(time.time()))
            return False
        print("ret true - token ok", file=sys.stderr)
        return True
        #return self.__expires > time.time()

    def __updateToken(self):
        pass

    def getNewToken(self):
        #print('get host', file=sys.stderr)
        self.logger.log(1, 'connector.__getNewToken', 'Retrieving new token')

        self.__host = self.__conf.getConfig()['host']
        #print('result: self.__host: ', self.__host, file=sys.stderr)

        #print('*** debug 1 ', file=sys.stderr)
        credentials = self.__conf.getCreddentials()
        data = {'grant_type': 'password',
                'client_id': 'invea-tech',
                'username': credentials['username'],
                'password': credentials['password']}
        #print('*** debug 2 ', file=sys.stderr)
        #print('self.host is ', self.__host ,  file=sys.stderr)
        #print('data: ', data, 'to url: ', self.__host + '/resources/oauth/token',  file=sys.stderr)
        r = requests.post(self.__host + '/resources/oauth/token', data=data, verify=False)
        #print('*** debug 3 ', file=sys.stderr)
        #print('---- Getting new token... ',r.json(), file=sys.stderr)

        self.__token = r.json()['access_token']
        self.__tokenType = r.json()['token_type']
        self.__expires = time.time() + r.json()['expires_in'] - 60
        print('now: ' + str(time.time()) + " exp:"+str(self.__expires), file=sys.stderr)
        self.logger.log(2, 'connector.__getNewToken', 'New token: '+self.__token +' Expires: '+str(self.__expires))
        self.logger.log(1, 'connector.__getNewToken', 'New token successfully retrieved')

    def sendGet(self, url, params="", data=""):
        self.__host = self.__conf.getConfig()['host']

        if self.__checkTokenExpiration() == False:
            #print('Token expired ', file=sys.stderr)
            self.getNewToken()

        #print(params, file=sys.stderr)
        #print(data, file=sys.stderr)

        self.logger.log(1, 'connector.sendGet', 'Sending GET to '+url)
        self.logger.log(2, 'connector.sendGet', 'Parameters: '+str(params)+' Data: '+str(data))

        headers = {'Authorization': self.__tokenType + ' ' + self.__token}
        r = requests.get(self.__host + url, params=params, data=data, headers=headers, verify=False)

        #print('HERE', file=sys.stderr)
        #print(r.url, file=sys.stderr)

        #print(r.json(), file=sys.stderr)

        self.logger.log(1, 'connector.sendGet', 'GET request to '+r.url+' returned with code '+str(r.status_code))
        if r.status_code == 500:
            self.logger.log(1, 'connector.sendGet', 'GET request finished with ERROR: ' + str(r.text))
        try:
            return r.json(), r.status_code
        except:
            return {}, r.status_code

    def sendPost(self, url, params="", data=""):
        self.__host = self.__conf.getConfig()['host']
        # check if token expired. If yes, update it
        if self.__checkTokenExpiration() == False:
            self.getNewToken()

        self.logger.log(1, 'connector.sendPost', 'Sending POST to ' + url)
        self.logger.log(2, 'connector.sendPost', 'Parameters: ' + str(params) + ' Data: ' + str(data))

        headers = {'Authorization': self.__tokenType + ' ' + self.__token}
        r = requests.post(self.__host + url, params=params, data=data, headers=headers, verify=False)

        self.logger.log(1, 'connector.sendGet', 'POST request to '+r.url+' returned with code ' + str(r.status_code))

        #print('HERE', file=sys.stderr)
        #print(r.url, file=sys.stderr)
        #print('HERE', file=sys.stderr)
        #print(r.json(), file=sys.stderr)
        try:
            return r.json(), r.status_code
        except:
            return {}, r.status_code

