from __future__ import print_function
import sys

__author__ = 'IBM'
from qpylib import qpylib
from app import app
from flask import render_template, request, Response, send_file
from connector.connector import Connector
from configurator.configurator import Configurator
import json, time, re, string, random
from datetime import datetime, timedelta
from logger.logger import Logger
from flask_wtf.csrf import CSRFProtect

logger = Logger()
conn = Connector()
conf = Configurator()

if conf.getConfig()['host'] != "":
    conn.getNewToken()

if conf.getConfig()['secretKey'] == "":
    alphabet = string.ascii_letters + string.digits
    while True:
        password = ''.join(random.choice(alphabet) for i in range(20))
        if (any(c.islower() for c in password)
                and any(c.isupper() for c in password)
                and sum(c.isdigit() for c in password) >= 3):
            break
    conf.setSecretKey(password)

app.secret_key = conf.getConfig()['secretKey']
app.config.update(
    SESSION_COOKIE_NAME='FlowmonApp-Session',
)
csrf = CSRFProtect(app)
csrf.init_app(app)

@app.before_request
def check_csrf():
     csrf.protect()


# because python2.6 doesn't have timedelta.total_seconds()
def timedelta_total_seconds(timedelta):
    return (timedelta.microseconds + 0.0 + (timedelta.seconds + timedelta.days * 24 * 3600) * 10 ** 6) / 10 ** 6

#print("Request to widget. Getting token:", conn.initialize(), file=sys.stderr)
# HTML routes
@app.route('/')
@app.route('/index')
def index():
    #print("Request to index. Getting token:", conn.initialize(), file=sys.stderr)
    if conn.initialize() is False:
        return render_template("notAuthorized.html")
    return render_template("index.html")


@app.route('/events')
def ev():
    return render_template("events.html")


@app.route('/flows')
def fl():
    return render_template("flows.html")


@app.route('/dashboard')
def db():
    return render_template("dashboard.html")


@app.route('/configuration')
def conf():
    return render_template("configuration.html")


# service routes
@app.route('/config')
def getConf():
    # with open('store/configuration.json') as conFile:
    #     config = json.load(conFile)
    # return json.dumps(config)
    conf = Configurator()
    return json.dumps(conf.getConfig())


@app.route('/config', methods=["POST"])
def setConf():
    newConf = request.json
    # with open('store/configuration.json', 'w') as conFile:
    #     json.dump(newConf, conFile)
    # return "ok"
    conf = Configurator()
    return conf.setConfig(newConf)


@app.route('/output_definitions')
def outDef():
    with open('store/outputDefinitions.json') as file:
        d = json.load(file)
    return json.dumps(d)


# api routes
@app.route('/api/perspectives')
def getPerspectives():
    resp = conn.sendGet('/rest/ads/perspectives')[0]
    return json.dumps(resp)


@app.route('/api/filters')
def getFilters():
    resp = conn.sendGet('/rest/ads/filters')[0]
    return json.dumps(resp)


@app.route('/api/methods')
def getMethods():
    resp = conn.sendGet('/rest/ads/methods')[0]
    return json.dumps(resp)


@app.route('/api/netflowSources')
def getFlowsources():
    resp = conn.sendGet('/rest/ads/net-flow-sources')[0]
    return json.dumps(resp)


@app.route('/api/events')
def getEvents():
    fr = request.args.get('from')
    to = request.args.get('to')
    source = request.args.get('source')
    target = request.args.get('target')
    type = request.args.get('type')
    filter = request.args.get('filter')
    persp = request.args.get('perspective')
    priority = request.args.get('priority')
    nfsource = request.args.get('nfsource')
    # create search object for flowmon api
    search = {}
    search['search'] = '{"from":"' + fr + '", "to":"' + to + '"}'
    if source != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"source": "' + source + '"}'

    if target != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"target": "' + target + '"}'

    if type != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"type": ' + type + '}'

    if filter != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"filter": ' + filter + '}'

    if persp != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"perspective": ' + persp + '}'

    if priority != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"priority": [' + priority + ']}'

    if nfsource != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"nfSource": ' + nfsource + '}'

    #print("Search: ", search, file=sys.stderr)

    search['limit'] = 10000

    resp = conn.sendGet('/rest/ads/events', search)[0]
    respjson = json.dumps({'data': resp})
    logger.log(1, 'endpoint getEvents', 'Received '+str(len(resp)) + ' events')
    logger.log(2, 'endpoint getEvents', 'Received events: '+str(resp))
    return respjson


@app.route('/api/events/<id>')
def getEventDetails(id):
    eventID = request.args.get('eventID')
    logger.log(1, 'endpoint getEventDetails', 'Retrieving event details for event ' + str(eventID))
    resp = conn.sendGet('/rest/ads/event/' + id)[0]
    return json.dumps(resp)


@app.route('/api/events/<id>/flows')
def getEventFlows(id):
    logger.log(1, 'endpoint getEventFlows', 'Retrieving flows for event ' + str(id))
    conf = Configurator()
    eventID = id
    resp = conn.sendGet('/rest/ads/event/' + eventID)[0]
    allProfiles = json.loads(getProfiles())
    #allChannels = []
    #print('resp:', resp,  file=sys.stderr)
    #for prof in allProfiles['data']:
        #print('prof :', prof, file=sys.stderr)
        #for ch in prof['channels']:
            #print('ch :', ch, file=sys.stderr)
            #allChannels.append(ch['id'])
    #print('allChannels :', allChannels, file=sys.stderr)
    #intersectChannels = [val for val in allChannels if val in resp['flowsSearch']['channels']]
    print("resp['flowsSearch']:", resp['flowsSearch'], file=sys.stderr)
    #print('\nintersectChannels :', intersectChannels, file=sys.stderr)
    search = {
        "from": resp['flowsSearch']['from'],
        "to": resp['flowsSearch']['to'],
        "profile": resp['flowsSearch']['profile'],
        "filter": resp['flowsSearch']['filter'],
        #"channels": resp['flowsSearch']['channels'],
        "channels": resp['flowsSearch']['channels'],
        "location": resp['flowsSearch']['location']
    }
    print("searchObj:", search, file=sys.stderr)
    output = conf.getConfig()['flowFormatInternal']

    # print('=======', file=sys.stderr)
    #print(search, file=sys.stderr)
    resp = conn.sendPost('/rest/fmc/analysis/flows',
                         data={"search": json.dumps(search), "output": json.dumps(output), "showonly": 10000})
    print('result', resp, file=sys.stderr)

    if (resp[1] != 200):
        errorMsg = resp[0]['message']
        pattern = re.compile("^Profile channel (.*) not found in profile .*$")
        m = pattern.match(errorMsg)
        if (m):
            return " Error: Misconfigured channel " + ','.join(str(i) for i in m.groups())  + " on Flowmon appliance.", 500
        return " Error: "+str(resp[0]['message']), 500

    try:
        searchID = resp[0]['id']
        logger.log(1, 'endpoint getEventFlows', 'Flow search id: ' + str(searchID))
        logger.log(1, 'endpoint getEventFlows', 'Checking if flow search has finished')
        resp = conn.sendGet('/rest/fmc/analysis/results/' + str(searchID))

        while True:
            if resp[1] == 200:
                logger.log(2, 'endpoint getEventFlows', 'Flow search has finished successfully')
                break
            logger.log(2, 'endpoint getEventFlows', 'Checking if flow search has finished')
            resp = conn.sendGet('/rest/fmc/analysis/results/' + str(searchID))

        # print('======= ',resp[1], file=sys.stderr)
        logger.log(1, 'endpoint getEventFlows', 'Received ' + str(len(resp[0])) + ' flows')
        logger.log(2, 'endpoint getEventFlows', 'Received flows: ' + str(resp[0]))
        return json.dumps(resp[0])
    except:
        logger.log(1, 'endpoint getEventFlows', 'Flow search has finished with ERROR: '+str(resp['message']))
        return " Error: "+str(resp['message']), 500


@app.route('/api/profiles')
def getProfiles():
    resp = conn.sendPost('/rest/fmc/getprofiles')[0]
    return json.dumps({'data': resp})


@app.route('/api/flows')
def getFlows():
    logger.log(1, 'endpoint getFlows', 'Retrieving flows')
    conf = Configurator()
    fr = request.args.get('from')
    to = request.args.get('to')
    profile = request.args.get('profile')
    channels = request.args.get('channels')
    listing = request.args.get('listing')
    filter = request.args.get('filter')
    count = request.args.get('count')
    search = {
        "from": fr,
        "to": to,
        "profile": profile,
        "channels": channels.split(','),
        "listing": listing,
        "filter": filter
    }
    # with open('store/configuration.json') as conFile:
    #     config = json.load(conFile)
    # output = ["ts","td","pr","sa","sp","da","dp","flg","tos","pkt","byt","pps","bps","bpp","fl"]
    # output = config['flowFormat']
    output = conf.getConfig()['flowFormatInternal']
    #print(search, file=sys.stderr)

    resp = conn.sendPost('/rest/fmc/analysis/flows',
                         data={"search": json.dumps(search), "output": json.dumps(output), "showonly": int(count)})
    #print('======= ', resp, file=sys.stderr)
    try:
        searchID = resp[0]['id']
        logger.log(1, 'endpoint getEventFlows', 'Flow search id: ' + str(searchID))
        logger.log(1, 'endpoint getFlows', 'Checking if flow search has finished')
        resp = conn.sendGet('/rest/fmc/analysis/results/' + str(searchID))
        while True:
            if resp[1] == 200:
                logger.log(1, 'endpoint getFlows', 'Flow search has finished successfully')
                break
            if resp[1] == 204:
                logger.log(1, 'endpoint getFlows', 'Flow search has finished successfully, but there are no flows')
                return json.dumps({}), 204
            logger.log(2, 'endpoint getFlows', 'Checking if flow search has finished')
            resp = conn.sendGet('/rest/fmc/analysis/results/' + str(searchID))
            # print('======= ', resp[0],resp[1], file=sys.stderr)

        logger.log(1, 'endpoint getFlows', 'Received ' + str(len(resp[0])) + ' flows')
        logger.log(2, 'endpoint getFlows', 'Received flows: ' + str(resp[0]))
        return json.dumps(resp[0])
    except:
        print('endpoint getEventFlows', 'Error: ' + str(resp), file=sys.stderr)
        if ('syntax error' in resp[0]['message']):
            logger.log(1, 'endpoint getEventFlows', 'Flow search has finished with ERROR: Bad filter (syntax error)')
            return "Error: "+"Bad filter", 400
        else:
            logger.log(1, 'endpoint getEventFlows', 'Flow search has finished with ERROR: ' + str(resp[0]['message']))
            return "Error: "+str(resp[0]['message']), 500


@app.route('/api/graph/type')
def graphType():
    aggrParts = 30
    fr = request.args.get('from')
    to = request.args.get('to')
    source = request.args.get('source')
    target = request.args.get('target')
    persp = request.args.get('perspective')
    priority = request.args.get('priority')
    nfsource = request.args.get('nfsource')
    filter = request.args.get('filter')
    type = request.args.get('type')  # events, sources, targets

    logger.log(1, 'endpoint graph', 'Starting graph routine')
    logger.log(2, 'endpoint graph', 'Parameters: from='+str(fr)+' to='+str(to)+' source='+str(source)+
               ' target='+str(target)+' perspective='+str(persp)+' priority='+str(priority)+
               ' nfsource='+str(nfsource) +' filter='+str(filter)+' type='+type)

    # frDT = timedelta_total_seconds(datetime.strptime(fr, '%Y-%m-%d %H:%M') - datetime(1970, 1, 1))
    # toDT = timedelta_total_seconds(datetime.strptime(to, '%Y-%m-%d %H:%M') - datetime(1970, 1, 1))
    # frDT -= 3600
    # toDT -= 3600

    frDT = datetime.strptime(fr, '%Y-%m-%d %H:%M')
    toDT = datetime.strptime(to, '%Y-%m-%d %H:%M')


    delta = timedelta_total_seconds(toDT - frDT)

    aggrDelta = int(delta / aggrParts)  # size of one aggrPart in seconds,
    tzoffset = time.timezone

    logger.log(2, 'endpoint graph', 'Counted aggregation delta in seconds: '+str(aggrDelta))

    # print("---- frDt:", frDT, datetime.fromtimestamp(int(frDT) + tzoffset).strftime("%Y/%m/%d %H:%M"), file = sys.stderr)
    # print("---- toDt:", toDT, datetime.fromtimestamp(int(toDT) + tzoffset).strftime("%Y/%m/%d %H:%M"), file = sys.stderr)

    # get events
    search = {}
    search['limit'] = 10000
    search['search'] = '{"from":"' + fr + '","to":"' + to + '"}'
    if source != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"source": "' + source + '"}'
    if target != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"target": "' + target + '"}'
    if persp != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"perspective": ' + persp + '}'
    if priority != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"priority": [' + priority + ']}'
    if nfsource != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"nfSource": ' + nfsource + '}'
    if filter != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"filter": ' + filter + '}'

    #print('   ***  Search object for graph data: ', search, file=sys.stderr)
    events,status = conn.sendGet('/rest/ads/events', search)
    if status == 500:
        if events['message'] == 'Token was not found.':
            return 'Error with token. Please reload application page', 427
    logger.log(1, 'endpoint graph', 'Received '+str(len(events))+' events')
    result = []
    for i in range(aggrParts):
        result.append({
            'from': (frDT + timedelta(0,aggrDelta * i)).strftime("%Y/%m/%d %H:%M:%S"),
            'to': (frDT + timedelta(0,aggrDelta * (i + 1))).strftime("%Y/%d/%m %H:%M:%S"),
            'stat': {}
        })
        # print(result[len(result)-1], file=sys.stderr)
    logger.log(2, 'endpoint graph', 'Created graph stats array: '+str(result))

    usedTypes = []
    # print('   ***  Number of returned events: ', len(events), file=sys.stderr)

    # because of bug - sometimes flowmon api returns empty data. Retry 2 times
    if len(events) == 0:
        logger.log(1, 'endpoint graph', 'Empty answer from API - try again')
        events = conn.sendGet('/rest/ads/events', search)[0]
        logger.log(1, 'endpoint graph', 'Received ' + str(len(events)) + ' events')
        result = []
        for i in range(aggrParts):
            result.append({
                'from': (frDT + timedelta(0,aggrDelta * i) ).strftime("%Y/%m/%d %H:%M:%S"),
                'to':(frDT + timedelta(0,aggrDelta * (i + 1) )).strftime("%Y/%d/%m %H:%M:%S"),
                'stat': {}
            })
            # print(result[len(result)-1], file=sys.stderr)
        usedTypes = []
        if len(events) == 0:
            logger.log(1, 'endpoint graph', 'Empty answer from API - try again')
            events = conn.sendGet('/rest/ads/events', search)[0]
            logger.log(1, 'endpoint graph', 'Received ' + str(len(events)) + ' events')
            result = []
            for i in range(aggrParts):
                result.append({
                    'from': (frDT + timedelta(0,aggrDelta * i) ).strftime("%Y/%m/%d %H:%M:%S"),
                    'to':(frDT + timedelta(0,aggrDelta * (i + 1) )).strftime("%Y/%d/%m %H:%M:%S"),
                    'stat': {}
                })
                # print(result[len(result)-1], file=sys.stderr)
            usedTypes = []
            if len(events) == 0:
                logger.log(1, 'endpoint graph', ' 3 times empty answer from API - return error')
                return 'Internal error', 502

    #print("Result after creation: ", result, file=sys.stderr)
    i = 1
    for event in events:
        logger.log(2, 'endpoint graph', 'Event '+str(i)+'/'+str(len(events))+': '+str(event))
        #eventTime = timedelta_total_seconds(datetime.strptime(event['time'], '%Y-%m-%d %H:%M:%S') - datetime(1970, 1, 1)) + tzoffset
        eventTime = datetime.strptime(event['time'], '%Y-%m-%d %H:%M:%S')
        offsetFromStart = eventTime - frDT
        offsetFromStart = timedelta_total_seconds(offsetFromStart)
        index = int(((  offsetFromStart   /  aggrDelta  )))
        logger.log(2, 'endpoint graph', 'Event ' + str(i) + '/' + str(len(events)) + ': eventTime='+str(eventTime)+
                   ' offsetFromStart='+str(offsetFromStart)+' index='+str(index))

        #print("--  Event type:", event['type'], " Event time: ", event['time'], " Index: ", index, file=sys.stderr)
        #print("Index: ", index, result[index], file=sys.stderr)

        if type == 'events':
            if event['type'] in result[index]['stat']:
                # print("incr", file=sys.stderr)
                result[index]['stat'][event['type']] += 1
            else:
                # print(" new", file=sys.stderr)
                result[index]['stat'][event['type']] = 1

        elif type == 'sources':
            if event['type'] in result[index]['stat']:
                # print("incr", file=sys.stderr)
                result[index]['stat'][event['type']].append(event['source']['ip'])
            else:
                # print(" new", file=sys.stderr)
                result[index]['stat'][event['type']] = [event['source']['ip']]

        elif type == 'targets':
            if event['type'] in result[index]['stat']:
                # print("incr", file=sys.stderr)
                result[index]['stat'][event['type']] += list(map(lambda x: x['ip'], event['targets']))
            else:
                # print(" new", file=sys.stderr)
                result[index]['stat'][event['type']] = list(map(lambda x: x['ip'], event['targets']))

        if event['type'] not in usedTypes:
            usedTypes.append(event['type'])

        i = i + 1

    for r in result:
        for t in usedTypes:
            if t not in r['stat']:
                r['stat'][t] = 0

    if type == 'targets' or type == 'sources':
        for o in result:
            for t in o['stat']:
                if o['stat'][t] != 0:
                    o['stat'][t] = len(set(o['stat'][t]))

    return json.dumps({'data': result})


@app.route('/api/graph/top_sources')
def topsou():
    fr = request.args.get('from')
    to = request.args.get('to')
    method = request.args.get('method')
    source = request.args.get('source')
    target = request.args.get('target')
    persp = request.args.get('perspective')
    priority = request.args.get('priority')
    nfsource = request.args.get('nfsource')
    filter = request.args.get('filter')

    logger.log(1, 'endpoint top_sources', 'Starting top_sources routine')
    logger.log(2, 'endpoint top_sources', 'Parameters: from=' + str(fr) + ' to=' + str(to) + ' source=' + str(source) +
               ' target=' + str(target) + ' perspective=' + str(persp) + ' priority=' + str(priority) +
               ' nfsource=' + str(nfsource) + ' filter=' + str(filter))

    search = {}
    search['limit'] = 10000
    search['search'] = '{"from":"' + fr + '","to":"' + to + '"}'
    if method != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"type": "' + method + '"}'
    if source != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"source": "' + source + '"}'
    if target != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"target": "' + target + '"}'
    if persp != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"perspective": ' + persp + '}'
    if priority != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"priority": [' + priority + ']}'
    if nfsource != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"nfSource": ' + nfsource + '}'
    if filter != None:
        search['search'] = search['search'][:-1]
        search['search'] += ',"filter": ' + filter + '}'

    events, status = conn.sendGet('/rest/ads/events', search)
    if status == 500:
        if events['message'] == 'Token was not found.':
            return 'Error with token. Please reload application page', 427

    logger.log(1, 'endpoint top_sources', 'Received '+str(len(events))+' events')

    stats = {}
    resolve = {}
    country = {}
    result = []
    # result = [
    #     {ip:"", count:},
    #     {ip:"", count:},
    #     {ip:"", count:},
    # ]
    i = 1
    for event in events:
        logger.log(2, 'endpoint top_sources', 'Event '+str(i)+'/'+str(len(events))+' source IP: '+str(event['source']['ip']))
        if event['source']['ip'] in stats:
            stats[event['source']['ip']] += 1
        else:
            stats[event['source']['ip']] = 1
            resolve[event['source']['ip']] = event['source']['resolved']
            country[event['source']['ip']] = event['source']['country']
        i = i+1
    for stat in stats:
        result.append({"ip": stat, "count": stats[stat], "resolved": resolve[stat], "country": country[stat]})

    return json.dumps({'data': result})


@app.route('/api/graph/priority')
def graphPrior():
    return ""


@app.route('/getcontext', methods=['GET'])
def get_context():
    context = request.args.get("context")
    qpylib.log("Setting the results to: " + context)
    return json.dumps({"app_id": qpylib.get_app_id(), "context": context})


@app.route('/widget', methods=['GET'])
def getDashboardContent():

    widgetId = request.args.get('id', type=int)
    widgetParams = Configurator().getWidgetsInfo()
    widgetPerspectiveObj = widgetParams[widgetId]

    logger.log(1, 'endpoint widget', 'Starting widget routine for widget ' + str(widgetId))
    logger.log(2, 'endpoint widget', 'Widget config:  params='+str(widgetParams)+' perspective='+str(widgetPerspectiveObj))

    conf = Configurator()
    if conf.getConfig()['host'] == "":
        return Response(response=json.dumps({'id': 'dashBoardItem1',
                                             'title': 'Flowmon Events by priority',
                                             'HTML': render_template('notAuthorizedWidget.html')}),
                        status=200, mimetype='application/json')


    try:
        perspectives_resp = conn.sendGet('/rest/ads/perspectives')
    except Exception as e:
        return Response(response=json.dumps({'id': 'dashBoardItem1',
                                             'title': 'Flowmon Events by priority',
                                             'HTML': render_template('notAuthorizedWidget.html')}),
                                             status=200, mimetype='application/json')
    if perspectives_resp[1] != 200:
        return Response(response=json.dumps({'id': 'dashBoardItem1',
                                             'title': 'Flowmon Events by priority',
                                             'HTML': render_template('notAuthorizedWidget.html')}),
                        status=200, mimetype='application/json')

    perspectives = perspectives_resp[0]

    #print('Widgets perspective', widgetPerspectiveObj['perspective_name'], widgetPerspectiveObj['perspective_id'], file=sys.stderr)
    #print('Perspectives: ', perspectives, file=sys.stderr)

    selectedPerspId = widgetPerspectiveObj['perspective_id']


    if widgetPerspectiveObj['perspective_id'] == 0:  # nothing is set in file
        selectedPerspId = perspectives[0]['id']
        widgetPerspectiveObj['perspective_id'] = perspectives[0]['name']
        widgetPerspectiveObj['perspective_name'] = perspectives[0]['name']
        Configurator().setWidgetInfoById(widgetId, perspectives[0]['id'], perspectives[0]['name'])
    if widgetPerspectiveObj['period'] == 0:  # nothing is set in file
        widgetPerspectiveObj['period'] = '1'
        Configurator().setWidgetPeriodById(widgetId, '1')

    search = {}
    if widgetPerspectiveObj['period'] == '1':
        fr = datetime.now() + timedelta(hours=-1)
        periodText = '1 hour'
    elif widgetPerspectiveObj['period'] == '3':
        fr = datetime.now() + timedelta(hours=-3)
        periodText = '3 hours'
    elif widgetPerspectiveObj['period'] == '6':
        fr = datetime.now() + timedelta(hours=-6)
        periodText = '6 hours'
    elif widgetPerspectiveObj['period'] == '12':
        fr = datetime.now() + timedelta(hours=-12)
        periodText = '12 hours'
    elif widgetPerspectiveObj['period'] == '24':
        fr = datetime.now() + timedelta(hours=-24)
        periodText = '24 hours'
    elif widgetPerspectiveObj['period'] == '72':
        fr = datetime.now() + timedelta(days=-3)
        periodText = '3 days'

    #fr = datetime.now() + timedelta(days=-1)
    fr = fr.strftime("%Y-%m-%d %H:%M:%S")
    to = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    #print('Data for widget:',fr, to,  file=sys.stderr)
    logger.log(2, 'endpoint widget', 'Widget'+str(widgetId)+' Timerange: from=' + str(fr) + ' to=' + str(to))
    #search['search'] = '{"from":"' + fr + '", "to":"' + to + '", "perspective":' + str(selectedPerspId) + '}'
    search['search'] = '{"from":"' + fr + '", "to":"' + to + '"}'
    if selectedPerspId != -1:
        search['search'] = search['search'][:-1] + ', "perspective":"' + str(selectedPerspId) + '"}'
    search['limit'] = 10000
    logger.log(1, 'endpoint widget', 'Get data for widget' + str(widgetId) + ': perspective ' + str(selectedPerspId) + "/" + widgetPerspectiveObj['perspective_name'] +
               " From: " + str(fr) + " To: " + str(to) + " (" + periodText + ")")
    resp,status = conn.sendGet('/rest/ads/events', search)
    if status == 500:
        if resp['message'] == 'Token was not found.':
            return 'Error with token. Please reload application page', 427

    logger.log(1, 'endpoint widget', 'Widget' + str(widgetId) + ' received '+str(len(resp))+' events')
    rules = ['', 'info', 'low', 'medium', 'high', 'critical']
    results = {
        'info': 0,
        'low': 0,
        'medium': 0,
        'high': 0,
        'critical': 0
    }
    for ev in resp:
        #if selectedPerspId != -1:

        if len(ev['perspectives']) == 0:
            continue
        secPersp = ''
        for evPersp in ev['perspectives']:
            if evPersp['id'] == selectedPerspId:
                secPersp = evPersp
                break
        # print('Event perspective', secPersp, file=sys.stderr)
        if secPersp == "":
            continue
        # print(rules[secPersp['priority']], ' += 1', file=sys.stderr)
        results[rules[secPersp['priority']]] += 1

        #else:
            #print('ev:', ev, file=sys.stderr)
            #results[rules[ev['priority']]] += 1

    #print('Results', results, file=sys.stderr)
    #print('Total events ', len(resp), file=sys.stderr)
    logger.log(1, 'endpoint widget', 'Widget' + str(widgetId) + ' data: ' + str(results))

    return Response(response=json.dumps({'id': 'dashBoardItem1',
                                         'title': 'Flowmon Events by priority',
                                         'HTML': render_template('widget.html',perspectives=perspectives,
                                                                 perspName=widgetPerspectiveObj['perspective_name'],
                                                                 period=periodText,
                                                                 crit=results['critical'], high=results['high'],
                                                                 med=results['medium'], low=results['low'],
                                                                 info=results['info'])}),
                    status=200, mimetype='application/json', )


@app.route('/widget_test', methods=['GET'])
def ggg():
    r = qpylib.REST('get', 'api/system/servers')
    #print('REST CALL', r, r.headers, file=sys.stderr)
    return json.dumps({'test': 'HOHOHO'})

@app.route('/widget/changePerspective', methods=['GET'])
def gggg():
    widgetId = request.args.get('id', type=int)
    newPerspId = request.args.get('perspid', type=int)

    #print('new persp val', newPerspId, type(newPerspId), file=sys.stderr)
    perspectives = conn.sendGet('/rest/ads/perspectives')[0]
    #print('perspectives', perspectives, file=sys.stderr)
    foundPersp = ''
    for persp in perspectives:
        #print('persp',persp, file=sys.stderr)
        if (persp['id'] == newPerspId ):
            foundPersp = persp
            break
    Configurator().setWidgetInfoById(widgetId, newPerspId, foundPersp['name'])
    return ""

@app.route('/widget/changePeriod', methods=['GET'])
def ggggg():
    widgetId = request.args.get('id', type=int)
    newPeriod = request.args.get('period', type=str)

    Configurator().setWidgetPeriodById(widgetId, newPeriod)
    return ""

@app.route('/get_log')
def getlog():
    return send_file('../store/log/flowmon_app.log', as_attachment=True,
                     attachment_filename='flowmon_app.log', cache_timeout=-1)
